from netbox.views import generic
from . import forms, models, tables
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect


class AppSystemView(generic.ObjectView):
    queryset = models.AppSystem.objects.all()

    def get_extra_context(self, request, instance):
        app_system_assignments = models.AppSystemAssignment.objects.filter(
            app_system=instance)
        assignments_table = tables.AppSystemAssignmentTable(
            app_system_assignments)
        assignments_table.columns.hide('app_system')
        assignments_table.configure(request)
        return {
            'assignments_table': assignments_table
        }


class AppSystemListView(generic.ObjectListView):
    queryset = models.AppSystem.objects.all()
    table = tables.AppSystemTable


class AppSystemEditView(generic.ObjectEditView):
    queryset = models.AppSystem.objects.all()
    form = forms.AppSystemForm


class AppSystemDeleteView(generic.ObjectDeleteView):
    queryset = models.AppSystem.objects.all()


class AppSystemAssignmentEditView(generic.ObjectEditView):
    queryset = models.AppSystemAssignment.objects.all()
    form = forms.AppSystemAssignmentForm
    template_name = 'netbox_app_systems/app_system_assignment_edit.html'

    def alter_object(self, instance, request, args, kwargs):
        if not instance.pk:
            # Assign the object based on URL kwargs
            content_type = get_object_or_404(
                ContentType, pk=request.GET.get('content_type'))
            instance.object = get_object_or_404(
                content_type.model_class(), pk=request.GET.get('object_id'))
        return instance

    def post(self, request, *args, **kwargs):
        form = forms.AppSystemAssignmentForm(request.POST)
        if form.is_valid():
            content_type_id = request.GET.get('content_type', -1)
            object_id = request.GET.get('object_id', -1)
            s = form.cleaned_data['app_system']
            qs = models.AppSystemAssignment.objects.filter(
                content_type=content_type_id, object_id=object_id, app_system=s.id)
            if qs.exists():
                redirect_url = request.GET.get('return_url', '/')
                return HttpResponseRedirect(redirect_url)

        return super().post(request, *args, **kwargs)


class AppSystemAssignmentDeleteView(generic.ObjectDeleteView):
    queryset = models.AppSystemAssignment.objects.all()



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


appsystem_buttons = [
    PluginMenuButton(
        link='plugins:netbox_app_systems:appsystem_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]


menu_items = (
    PluginMenuItem(
        link='plugins:netbox_app_systems:appsystem_list',
        link_text='App Systems',
        buttons=appsystem_buttons
    ),
)



from django.db.models import Count

from netbox.api.viewsets import NetBoxModelViewSet

from .. import models
from .serializers import AppSystemSerializer, AppSystemAssignmentSerializer
from .. import filtersets

class AppSystemViewSet(NetBoxModelViewSet):
    queryset = models.AppSystem.objects.prefetch_related('tags')
    serializer_class = AppSystemSerializer


class AppSystemAssignmentViewSet(NetBoxModelViewSet):
    queryset = models.AppSystemAssignment.objects.prefetch_related(
        'object', 'app_system')
    serializer_class = AppSystemAssignmentSerializer
    filterset_class = filtersets.AppSystemAssignmentFilterSet



from attr import fields
from django import forms
from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField
from .models import AppSystem, AppSystemAssignment
from utilities.forms import (
    BootstrapMixin, DynamicModelChoiceField)


class AppSystemForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = AppSystem
        fields = ('name', 'slug', 'description', 'comments', 'tags')


class AppSystemAssignmentForm(BootstrapMixin, forms.ModelForm):
    app_system = DynamicModelChoiceField(queryset=AppSystem.objects.all())

    class Meta:
        model = AppSystemAssignment
        fields = ('app_system',)



from extras.plugins import PluginTemplateExtension
from django.contrib.contenttypes.models import ContentType
from .models import AppSystemAssignment
# from netbox.models.


class AppSystemVMPanel(PluginTemplateExtension):
    model = 'virtualization.virtualmachine'
    # model = 'dcim.device'

    def left_page(self):
        vm = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=vm).id
        app_systems = AppSystemAssignment.objects.filter(
            object_id=vm.id, content_type=content_type_id)
        # print(vars(AppSystem_ass))
        # print(AppSystem_ass)
        AppSystems = []
        for s in app_systems:
            AppSystems.append({
                'id': s.id,
                'app_system': s.app_system})
            # print(s.__dict__)

        print(AppSystems)
        return self.render('netbox_app_systems/app_system_panel.html', extra_context={
            'app_systems': AppSystems
        })


class AppSystemDevicePanel(PluginTemplateExtension):
    model = 'dcim.device'

    def left_page(self):
        vm = self.context['object']
        content_type_id = ContentType.objects.get_for_model(model=vm).id
        app_systems = AppSystemAssignment.objects.filter(
            object_id=vm.id, content_type=content_type_id)
        AppSystems = []
        for s in app_systems:
            AppSystems.append({
                'id': s.id,
                'app_system': s.app_system})
            # print(s.__dict__)

        return self.render('netbox_app_systems/app_system_panel.html', extra_context={
            'app_systems': AppSystems
        })


template_extensions = [AppSystemVMPanel, AppSystemDevicePanel]



from email.policy import default
from tabnanny import verbose
import django_tables2 as tables

from netbox.tables import NetBoxTable, columns
from .models import AppSystem, AppSystemAssignment


class AppSystemTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = AppSystem
        fields = ('pk', 'id', 'name', 'description', 'comments')
        default_columns = ('name', 'description')


class AppSystemAssignmentTable(NetBoxTable):
    content_type = columns.ContentTypeColumn(verbose_name='Object type')
    object = tables.Column(linkify=True, orderable=False)
    app_system = tables.Column(linkify=True)
    actions = columns.ActionsColumn(actions=('edit', 'delete'))

    class Meta(NetBoxTable.Meta):
        model = AppSystemAssignment
        fields = ('pk', 'content_type', 'object', 'app_system', 'actions')
        default_columns = ('pk', 'content_type', 'object', 'app_system')



import django_filters
from netbox.filtersets import ChangeLoggedModelFilterSet, NetBoxModelFilterSet
from utilities.filters import ContentTypeFilter
from .models import *
from django.db.models import Q


class AppSystemFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = AppSystem
        fields = ['id', 'name', 'slug', 'description', 'comments']

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(slug__icontains=value) |
            Q(description__icontains=value) |
            Q(comments__icontains=value)
        )


class AppSystemAssignmentFilterSet(ChangeLoggedModelFilterSet):
    content_type = ContentTypeFilter()
    app_system_id = django_filters.ModelMultipleChoiceFilter(
        queryset=AppSystem.objects.all(),
        label='AppSystem (ID)',
    )

    class Meta:
        model = AppSystemAssignment
        fields = ['id', 'content_type_id', 'object_id']



from genericpath import exists
from django.db import models
from django.urls import reverse

from netbox.models import NetBoxModel
from netbox.models import ChangeLoggedModel
from netbox.models.features import WebhooksMixin
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey


class AppSystem(NetBoxModel):
    name = models.CharField(max_length=200)
    slug = models.SlugField(max_length=100, unique=True)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_app_systems:appsystem', args=[self.pk])

    class Meta:
        ordering = ['name']


class AppSystemAssignment(WebhooksMixin, ChangeLoggedModel):
    content_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE
    )
    object_id = models.PositiveBigIntegerField()
    object = GenericForeignKey(
        ct_field='content_type',
        fk_field='object_id'
    )
    app_system = models.ForeignKey(
        to='netbox_app_systems.AppSystem',
        on_delete=models.PROTECT,
        related_name='appsystem_assignments'
    )

    clone_fields = ('content_type', 'object_id')

    class Meta:
        ordering = ['app_system']
        unique_together = ('content_type', 'object_id',
                           'app_system')

    def __str__(self):
        return str(self.app_system)

    def get_absolute_url(self):
        return reverse('plugins:netbox_app_systems:appsystem', args=[self.app_system.pk])



from netbox.api.routers import NetBoxRouter
from . import views


app_name = 'netbox_app_systems'

router = NetBoxRouter()
router.register('app-systems', views.AppSystemViewSet)
router.register('appsystem-assignment', views.AppSystemAssignmentViewSet)

urlpatterns = router.urls



from rest_framework import serializers

from ipam.api.serializers import NestedPrefixSerializer
from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from ..models import AppSystem, AppSystemAssignment
from drf_yasg.utils import swagger_serializer_method
from django.contrib.auth.models import ContentType
from netbox.api import ContentTypeField
from utilities.api import get_serializer_for_model


class NestedAppSystemSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_app_systems-api:appsystem-detail')

    class Meta:
        model = AppSystem
        fields = ('id', 'slug', 'url', 'display', 'name')


class AppSystemSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_app_systems-api:appsystem-detail')

    class Meta:
        model = AppSystem
        fields = ('id', 'slug', 'url', 'display', 'name', "description",
                  'comments', 'tags', 'custom_fields', 'created', 'last_updated')


class NestedAppSystemAssignmentSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_app_systems-api:appsystemassignment-detail')
    app_system = NestedAppSystemSerializer()

    class Meta:
        model = AppSystemAssignment
        fields = ['id', 'url', 'display', 'AppSystem', 'relation']


class AppSystemAssignmentSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_app_systems-api:appsystemassignment-detail')
    content_type = ContentTypeField(
        queryset=ContentType.objects.all()
    )
    object = serializers.SerializerMethodField(read_only=True)
    app_system = NestedAppSystemSerializer()

    class Meta:
        model = AppSystemAssignment
        fields = [
            'id', 'url', 'display', 'content_type', 'object_id', 'object', 'app_system', 'created',
            'last_updated',
        ]

    @swagger_serializer_method(serializer_or_field=serializers.DictField)
    def get_object(self, instance):
        serializer = get_serializer_for_model(
            instance.content_type.model_class(), prefix='Nested')
        context = {'request': self.context['request']}
        return serializer(instance.object, context=context).data



from extras.plugins import PluginConfig


class NetBoxAppSystemConfig(PluginConfig):
    name = 'netbox_app_systems'
    verbose_name = 'Application Systems'
    description = 'Netbox plugin. Assign devices and virtual machines to application systems'
    version = '0.1'
    author = 'Oleg Senchenko'
    author_email = 'senchenkoob@mail.ru'
    base_url = 'app-systems'


config = NetBoxAppSystemConfig



# Generated by Django 4.0.4 on 2022-05-26 08:16

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('extras', '0073_journalentry_tags_custom_fields'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppSystem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(max_length=200)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='AppSystemAssignment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveBigIntegerField()),
                ('app_system', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='appsystem_assignments', to='netbox_app_systems.appsystem')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'ordering': ['app_system'],
                'unique_together': {('content_type', 'object_id', 'app_system')},
            },
        ),
    ]



from attr import fields
from graphene import ObjectType
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.fields import ObjectField, ObjectListField
from . import filtersets, models


class AppSystemType(NetBoxObjectType):
    class Meta:
        model = models.AppSystem
        fields = '__all__'


class AppSystemAssignmentType(NetBoxObjectType):
    class Meta:
        model = models.AppSystemAssignment
        fields = '__all__'
        filterset_class = filtersets.AppSystemAssignmentFilterSet


class Query(ObjectType):
    app_system = ObjectField(AppSystemType)
    app_system_list = ObjectListField(AppSystemType)
    app_system_assignment = ObjectField(AppSystemAssignmentType)
    app_system_assignment_list = ObjectListField(AppSystemAssignmentType)


schema = Query



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='netbox-app-systems',
    version='0.2.4',
    description='Netbox plugin. Assign devices and virtual machines to application systems',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[],
    download_url='https://pypi.org/project/netbox-app-systems/',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    keywords=['netbox', 'netbox-plugin'],
    author='Oleg Senchenko',
    author_email='senchenkoob@mail.ru',
    maintainer='Oleg Senchenko',
    maintainer_email='senchenkoob@mail.ru',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



from django.urls import path

from netbox.views.generic import ObjectChangeLogView
from . import models, views

urlpatterns = (
    path('app-systems/', views.AppSystemListView.as_view(), name="appsystem_list"),
    path('app-systems/add', views.AppSystemEditView.as_view(), name="appsystem_add"),
    path('app-systems/<int:pk>/',
         views.AppSystemView.as_view(), name="appsystem"),
    path('app-systems/<int:pk>/edit',
         views.AppSystemEditView.as_view(), name="appsystem_edit"),
    path('app-systems/<int:pk>/delete',
         views.AppSystemDeleteView.as_view(), name="appsystem_delete"),
    path('app-systems/<int:pk>/changelog', ObjectChangeLogView.as_view(),
         name="appsystem_changelog", kwargs={'model': models.AppSystem}),

    # app system assignment
    path('app-system-assignment/add/', views.AppSystemAssignmentEditView.as_view(),
         name='appsystemassignment_add'),
    path('app-system-assignment/<int:pk>/edit/', views.AppSystemAssignmentEditView.as_view(),
         name='appsystemassignment_edit'),
    path('app-system-assignment/<int:pk>/delete/', views.AppSystemAssignmentDeleteView.as_view(),
         name='appsystemassignment_delete'),
)


# path('app-system/', views.AppSystem.as_view(), name="appsystem_"),
